import logging
import os
from invirtualenv.config import generate_parsed_config_file
from invirtualenv.plugin_base import InvirtualenvPlugin


logger = logging.getLogger(__name__)


class InvirtualenvParsedConfig(InvirtualenvPlugin):
    package_formats = ['parsed_deploy_conf']
    default_config_filename = 'deploy.conf.parsed'
    package_template = None

    def __init__(self, *args, **kwargs):
        super(InvirtualenvParsedConfig, self).__init__(*args, **kwargs)

        if os.path.exists(self.config_file):
            with open(self.config_file) as config_file_handle:
                self.package_template = config_file_handle.read()
            logger.debug('Read template %r', self.package_template)

    def run_package_command(self, package_hashes, wheel_dir='wheels'):
        if not os.path.exists(self.config_file):
            raise FileNotFoundError('The invirtualenv configuration file %r was not found' % self.config_file)
        generate_parsed_config_file('deploy.conf', self.default_config_filename)
        return self.default_config_filename




# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
Exceptions generated by the invirtualenv module
"""

class InvirtualenvError(Exception):
    """
    An invirtualenv error occurred.
    """
    pass

class AlreadyExists(InvirtualenvError):
    """
    Package already Exists exception
    """
    pass


class BuildException(InvirtualenvError):
    """
    There was an error Building the VirtualEnv
    """
    pass


class CommandNotFound(InvirtualenvError):
    """
    The command specified wasn't found
    """
    pass


class InsufficientPermissions(InvirtualenvError):
    """
    The current user has insufficient permissions to complete the specified
    operation.
    """
    pass


class NoPackageVersions(InvirtualenvError):
    """
    Query of the Python PyPi repo returned no packages with the specified
    name.
    """
    pass


class PackageConfigFailure(InvirtualenvError):
    """
    Unable to create a package configuration file
    """
    pass


class PackageGenerationFailure(InvirtualenvError):
    """
    Package generation failed
    """
    pass



# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
InVirtualEnv Python Virtualenv Installer Module
"""
try:
    import pkg_resources
    __version__: str = pkg_resources.get_distribution("invirtualenv").version
except ImportError:
    ___version__ = '0.0.0'

__copyright__ = "Copyright 2016, Yahoo Inc."
__all__ = [
    'config',
    'contextmanager',
    'deploy',
    'exceptions',
    'package',
    'plugin',
    'plugin_base',
    'utility',
    'virtualenv'
]



# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
Functions for managing packaging
"""
from collections import defaultdict
from distutils.version import LooseVersion, StrictVersion
from html.parser import HTMLParser
import logging
import os
import platform
import subprocess  # nosec
from typing import DefaultDict, Dict, List, Optional

try:  # pragma: no cover
    import configparser as ConfigParser
except ImportError:  # pragma: no cover
    import ConfigParser

from defusedxml import ElementTree
import pkg_resources
import requests

from .exceptions import BuildException
from .utility import display_header
from distutils.version import LooseVersion


logger = logging.getLogger(__name__)


class HTMLLinkParser(HTMLParser):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.href_list = []

    def handle_starttag(self, tag, attrs):
        if tag == 'a':
            for name, value in attrs:
                if name == "href":
                    self.href_list.append(value)


def get_index_url():
    index_url = 'https://pypi.python.org/simple'
    if os.path.exists('/etc/pip.conf'):  # pragma: no cover
        cfg = ConfigParser.SafeConfigParser({'index-url': index_url})
        cfg.read('/etc/pip.conf')
        index_url = cfg.get('global', 'index-url')
    return index_url.rstrip('/')


def package_files(package: str, pypi_url: str='https://pypi.org') -> List[str]:
    if not pypi_url:
        pypi_url = get_index_url()
    name = pkg_resources.safe_name(package)
    url = pypi_url + '/simple/' + package + '/'
    parser = HTMLLinkParser()

    # Stream the index
    req = requests.get(url, stream=True)
    for line in req.iter_lines():
        line = line.decode(errors='ignore')
        parser.feed(line)

    files = []
    for filename in parser.href_list:
        filename = os.path.basename(filename).split('#')[0]
        files.append(filename)
    return files


def package_type_versions(package: str, pypi_url: str='https://pypi.org', require_strict: bool=False) -> Dict[str, list]:
    if not pypi_url:
        pypi_url = get_index_url()

    package_releases: DefaultDict[str, List] = defaultdict(lambda: [], {'tar.gz': [], 'whl': [], 'zip': []})
    name = pkg_resources.safe_name(package)

    sorted_key = StrictVersion if require_strict else LooseVersion

    for filename in package_files(package=package, pypi_url=pypi_url):
        # Separate the releases by extension
        for ext in package_releases.keys():  # pragma: no cover
            if filename.endswith('.' + ext):
                filename = filename[len(name) +1 :-(len(ext) + 1)]
                if ext == 'whl':
                    split_name = filename.split('-')
                    filename = '-'.join(split_name[:-3])
                    tags = '-'.join(split_name[-3:])
                    package_releases[f'{ext}-{tags}'].append(filename)
                package_releases[ext].append(filename)
                break

    # Sort the results
    for value in package_releases.values():
        try:
            sorted(value, key=sorted_key)
        except ValueError as error:
            if not require_strict:
                logger.info(f'Invalid package version {value!r} found in the index using strict versioning, falling back to loose versioning')
                sorted( value, key=LooseVersion)
            else:
                raise error
    return package_releases


def package_versions(package: str, pypi_url: str='https://pypi.org', require_strict: bool=False) -> List[str]:
    if not pypi_url:
        pypi_url = get_index_url()

    all_versions = set()
    for key, value in package_type_versions(package, pypi_url=pypi_url, require_strict=require_strict).items():
        all_versions.update(value)
    return list(all_versions)


def existing_package(
        package: str, version: str, package_types: Optional[List[str]]=None, pypi_url: str='https://pypi.org'
) -> bool:
    if not pypi_url:
        pypi_url = get_index_url()

    if not package_types:  # pragma: no cover
        package_types = ['tar.gz', 'whl', 'zip']

    package_releases = package_type_versions(package, pypi_url=pypi_url)

    for package_type in package_types:
        if package_type not in package_releases.keys():
            continue
        versions = list(package_releases[package_type])
        if version in versions:
            return True

    return False


def package_scripts_directory():
    """
    Get the package scripts directory

    Returns
    str
        The path to the directory containing the package scripts
    """
    module_directory = os.path.dirname(__file__)
    script_directory = os.path.join(
        module_directory,
        'package_scripts'
    )
    return script_directory


def strip_from_end(text, suffix):
    """
    Strip a substring from the end of a string

    Parameters
    ----------
    text : str
        The string to be evaluated

    suffix : str
        The suffix or substring to remove from the end of the text string

    Returns
    -------
    str
        A string with the substring removed if it was found at the end of the
        string.
    """
    if not text.endswith(suffix):
        return text
    return text[:len(text)-len(suffix)]


def latest_package_version(package):
    """
    Get the latest version number for a package

    Parameters
    ----------
    package : str
        Package to get the latest version of

    Returns
    -------
    str
        Latest package version or an empty string if no versions are found
    """
    versions = package_versions(package)
    if not versions:
        return ''
    return versions[-1]


def install_prereq_packages(test=False):  # pragma: no cover
    """
    Install packages required to build python

    Parameters
    ----------
    test: bool, optional
        Don't run the actual command, if True
    """
    resulting_files = [
        '/usr/bin/gcc',
        '/usr/bin/make'
    ]
    needed_packages = [
        'gcc',
        'make',
        'git',
        'libffi',
        'libffi-devel',
        'gdb',
        'glibc-devel',
        'libstdc++-devel',
        'libcom_err-devel',
        'krb5-devel',
        'openssl-devel',
        'gettext-devel',
        'gdbm-devel',
        'zlib-devel',
        'libsepol-devel',
        'sqlite-devel',
        'tk-devel',
        'expat-devel',
        'readline-devel',
        'ncurses-devel',
        'bzip2-devel',
        'xz-devel',
        'gmp-devel',
        'mysql-devel',

        # Packages not in RHEL releases before RHEL6
        'db4-devel',
        'libpcap-devel',
    ]
    if platform.system() in ['Darwin']:
        print('No prereq packages defined for platform %s' % platform.system())
        return
    redhat_release = '7.0'
    if os.path.exists('/etc/redhat-release'):
        redhat_release = open('/etc/redhat-release').readlines()[0].strip().split()[-2]
    display_header('Installing build system needed for build on Redhat %s' % redhat_release)
    display_header('Installing additional build dependencies')
    if not test:  # pragma: no cover
        subprocess.check_output(['yum', '-y', 'install'] + needed_packages)  # nosec

    display_header('Verifying needed dependencies where installed')
    for filename in resulting_files:
        if not test and not os.path.exists(filename):  # pragma: no cover
            raise BuildException('Required file %s is missing' % filename)

    if redhat_release.startswith('6'):
        display_header('Fixing the user .gitconfig to work with git < 1.8')
        gitconfig = os.path.expanduser('~/.gitconfig')
        if os.path.exists(gitconfig):
            old_git_config = []
            with open(gitconfig) as file_handle:
                old_git_config = file_handle.read().split('\n')

            with open(gitconfig, 'w') as file_handle:
                for line in old_git_config:
                    if 'default=simple' not in line.replace(' ', ''):
                        file_handle.write(line + '\n')



#!/usr/bin/env python3
from __future__ import print_function
import json
import logging
import os
import shutil
import sys
from invirtualenv.deploy import unlink_deployed_bin_files
from invirtualenv.config import get_configuration_dict


if __name__ == "__main__":
    log_level = logging.INFO
    if os.environ.get('RPM_SCRIPTLET_DEBUG', 'false').lower() in ['true', '1', 'on']:
        log_level = logging.DEBUG

    logging.basicConfig(level=log_level, format='%(asctime)s %(levelname)-5s [%(filename)s:%(lineno)d] %(message)s', datefmt='%Y%m%d:%H:%M:%S')

    logger = logging.getLogger(os.path.basename(sys.argv[0]))

    upgrade = False
    scriptlet_argument = os.environ.get('RPM_ARG', '')
    if scriptlet_argument and scriptlet_argument == '1':
        upgrade = True
    logger.debug('Running pre_uninstall %s, upgrade=%r', scriptlet_argument, upgrade)
    # logger.debug('Environment: %s' % json.dumps(dict(os.environ), indent=4))


    this_script_dir = os.path.dirname(os.path.realpath(__file__))
    path_bits = this_script_dir.split(os.path.sep)

    # Remove leading package_scripts from the path
    path_bits.remove('package_scripts')
    data_dir = os.path.sep.join(path_bits)
    deploy_conf = os.path.join(data_dir, 'deploy.conf')
    if not os.path.exists(deploy_conf):
        print("No 'deploy.conf' found.  Doing nothing", file=sys.stderr)
        sys.exit(0)

    config = get_configuration_dict(deploy_conf)
    venv_dir = config['global'].get('virtualenv_deploy_dir', None)

    if venv_dir and os.path.exists(venv_dir):
        if upgrade:
            logger.debug('Package upgrade is running, not deleting bin files to prevent removing links from the new package')
        else:
            unlink_deployed_bin_files(venv_dir)
        logger.debug('Removing virtualenv directory %r' % venv_dir)
        shutil.rmtree(venv_dir)



# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
Functions to enable packaging plugin functionality
"""
import logging
import os
import shutil
import subprocess  # nosec
import sys
from jinja2 import Template
from . import __version__
from .config import get_configuration_dict, get_configuration, generate_parsed_config_file
from .contextmanager import InTemporaryDirectory, working_dir
from .utility import find_executable, update_recursive, csv_list


logger = logging.getLogger(__name__)  # pylint: disable=C0103


class InvirtualenvPlugin(object):
    package_formats = []
    config_default = ""
    config_types = {}
    default_config_filename = 'invirtualenv.configuration'
    package_template = ''
    hash = None  # PIP hash algorithm to use, can be sha256, sha384, sha512 or None (no hashing)
    noarch = True

    def __init__(self, config_file='deploy.conf', source_dir=''):
        self._wheel_hashes = {}
        self.config_file = config_file
        self.config = get_configuration_dict(configuration=config_file)
        self.source_dir = source_dir if source_dir else os.getcwd()
        self.loaded_configuration = get_configuration(configuration=config_file)
        self.add_plugin_configuration()

    # Methods that need to be written for each plugin type
    def run_package_command(self, package_hashes, wheel_dir='wheels'):
        """
        Run the command to generate the package based on the hash
        """
        pass

    def system_requirements_ok(self):
        """
        Check if all the system requirements for this plugin are met.

        Returns
        -------
        bool
            True if requirements are met, False otherwise
        """
        return True

    @property
    def basepython(self):
        """
        Determine the value of basepython for this plugin

        Returns
        -------
        str: basepython
        """
        basepython = self.config['global'].get('basepython', 'python3')

        # If basepython is set in the packages section it needs to override the global default
        for format in self.package_formats:
            pkg_cfg = self.config.get(format + "_package", {})
            if basepython in pkg_cfg.keys():
                print('Using basepython from %s_package section' % format)
                basepython = pkg_cfg['basepython']


        return basepython

    @property
    def pip_cmd(self):
        """
        Get the pip command used to create wheels of packages.

        The intention is to use the same version of pip to build the wheels
        as would be used to deploy them.

        The full path to the python interpreter is used to avoid shebang
        line length issues.

        Returns
        -------
        list
            Command that can be used to invoke pip
        """
        python_executable = find_executable(self.basepython)
        if not python_executable:
            python_executable = sys.executable
        bin_dir = os.path.dirname(python_executable)
        try:
            output = subprocess.check_output([python_executable, '-m', 'pip'])  # nosec
            return [python_executable, '-m', 'pip']
        except subprocess.CalledProcessError:
            # Try to work around broken pip module
            pip_exe = os.path.join(bin_dir, 'pip3')
            if os.path.exists(pip_exe):
                return [pip_exe]
            return [os.path.join(bin_dir, 'pip')]

    def supported_formats(self):
        """
        Formats supported by this plugin that can be generated on this system

        Returns
        -------
        list
            Package formats that this plugin supports
        """
        if self.system_requirements_ok():
            return self.package_formats
        return []

    def copy_files_to_tempdir(self, tempdir):
        """
        Copy any files from the sourcedir into the tempdir for package generation
        """
        pass

    def create_package(self, package_type):
        """
        Generate a package of the specified type

        Parameters
        ----------
        str: package_type
            The type of package to generate
        """
        if package_type not in self.supported_formats():
            return None

        use_local_wheels = self.config['global'].get('use_local_wheels', 'false').lower() in ['1', 'true', 'yes', 'on']
        include_hashes = self.config['pip'].get('hash_dependencies', 'false').lower() in ['1', 'true', 'yes', 'on']

        original_directory = os.getcwd()

        with InTemporaryDirectory() as tempdir:
            self.copy_files_to_tempdir(tempdir)

            wheel_dir = 'wheels'
            os.makedirs(wheel_dir)
            hashes = self.generate_wheel_packages(wheel_dir)
            self.generate_wheel_archive()
            deps = []
            for package_name, package_hash in hashes.items():
                if include_hashes:
                    deps.append('{package_name} --hash={package_hash}'.format(package_name=package_name, package_hash=package_hash))
                else:
                    deps.append(package_name)
            self.config['pip']['deps'] = deps
            if self.hash:
                self.loaded_configuration['pip']['deps'] = '\n'.join(deps)
            with open('deploy.conf.unparsed', 'w') as deploy_conf_handle:
                self.loaded_configuration.write(deploy_conf_handle)
            with open('deploy.conf.unparsed') as fh:
                logger.debug('deploy.conf.unparsed %s', fh.read())
            generate_parsed_config_file('deploy.conf.unparsed', 'deploy.conf')
            package = self.run_package_command(hashes, wheel_dir=wheel_dir)  # pylint: disable=E1128,E1111
            if package and os.path.exists(package):
                source = package
                dest = os.path.join(original_directory, os.path.basename(package))
                shutil.copyfile(source, dest)
                return dest
            return package

    def generate_wheel_archive(self, filename=None):
        if not filename:
            filename = 'wheels.tar.gz'
        subprocess.check_call(['tar', '-czf', filename, 'wheels'])  # nosec

    def generate_wheel_packages(self, wheeldir):
        """
        Generate wheel packages for all dependencies

        Parameters
        ----------
        wheeldir: str
            The directory path to store the generated wheel packages

        Returns
        -------
        dict of filename, pip requirements line
        """
        if not self.config['pip'].get('deps'):
            return {}
        hashes = {}

        with working_dir(wheeldir):
            logger.debug('Making sure the wheel package is installed')
            subprocess.check_call(self.pip_cmd + ['install', '-U', 'pip'])  # nosec
            subprocess.check_call(self.pip_cmd + ['install', 'wheel'])  # nosec
            deps = self.config['pip'].get('deps', []) + ['invirtualenv', 'configparser']
            cmd = self.pip_cmd + ['wheel', '-w', '.'] + deps
            logger.debug('Running pip command %r to generate wheel packages', cmd)
            try:
                output = subprocess.check_output(cmd, stderr=subprocess.STDOUT).decode()  # nosec
            except subprocess.CalledProcessError as error:
                logger.warning('Exception occurred while generating wheel packages, downloading source packages')
                if error.stdout:
                    logger.error(error.stdout.decode())
                if error.stderr:
                    logger.error(error.stderr.decode())
                cmd = self.pip_cmd + ['download', '-d', '.'] + deps
                logger.debug('Running pip command %r to download missing packages', cmd)
                try:
                    output = subprocess.check_output(cmd, stderr=subprocess.STDOUT).decode()  # nosec
                except subprocess.CalledProcessError as error:
                    logger.warning('Exception occurred while downloading source packages')
                    if error.stdout:
                        logger.error(error.stdout.decode())
                    if error.stderr:
                        logger.error(error.stderr.decode())
                    raise
            for filename in os.listdir('.'):
                if filename.endswith('.whl'):
                    split_filename = os.path.basename(filename).split('-')
                    file_wheel_name = filename
                    file_wheel_version = None
                    if len(split_filename) > 2:
                        file_wheel_name = split_filename[0]
                        file_wheel_version = split_filename[1]
                    if self.noarch and not filename.endswith('none-any.whl'):
                        self.noarch = False
                    cmd = self.pip_cmd + ['hash']
                    if self.hash:
                        cmd += ['-a', self.hash]
                    cmd += [filename]
                    logger.debug('Running pip command %r to generate package hash for %r', cmd, filename)
                    hash_result = subprocess.check_output(cmd).decode()  # nosec
                    if file_wheel_name and file_wheel_version:
                        logger.debug("{file_wheel_name}=={file_wheel_version}".format(**locals()))
                        hashes['{file_wheel_name}=={file_wheel_version}'.format(**locals())] = '='.join(hash_result.split(os.linesep)[1].split('=')[1:])
                        logger.debug('Got requirements line %r', hashes['{file_wheel_name}=={file_wheel_version}'.format(**locals())])
                    else:
                        hashes[filename] = '='.join(hash_result.split(os.linesep)[1].split('=')[1:])  # nosec
                        logger.debug('Got requirements line %r', hashes[filename])
        self._wheel_hashes = hashes
        self.add_plugin_configuration()
        return hashes

    def add_plugin_configuration(self):
        """
        Add any specific plugin configuration values to the configuration
        :return:
        """
        pass

    def render_template_with_config(self, template_str=''):

        if not template_str:
            template_str = self.package_template

        original_directory = os.getcwd()

        use_local_wheels = self.config['global'].get('use_local_wheels', 'false').lower() in ['1', 'true', 'yes', 'on']
        include_hashes = self.config['pip'].get('hash_dependencies', 'false').lower() in ['1', 'true', 'yes', 'on']

        with InTemporaryDirectory() as tempdir:
            self.copy_files_to_tempdir(tempdir)
            if self._wheel_hashes:
                hashes = self._wheel_hashes
            else:
                wheel_dir = 'wheels'
                os.makedirs(wheel_dir, exist_ok=True)
                hashes = self.generate_wheel_packages(wheel_dir)

            deps = []
            for package_name, package_hash in hashes.items():
                if use_local_wheels and include_hashes:
                    deps.append('{package_name} --hash={package_hash}'.format(package_name=package_name, package_hash=package_hash))
                else:
                    deps.append(package_name)
            self.config['pip']['deps'] = deps

            if use_local_wheels or include_hashes:
                self.loaded_configuration['pip']['deps'] = '\n'.join(deps)

        template = Template(template_str)
        return template.render(self.config)



from invirtualenv.config import get_configuration
from invirtualenv.deploy import link_deployed_bin_files



def main():
    config = get_configuration()
    link_bin_files = config['docker_container'].get('link_bin_files', '')
    if link_bin_files:
        link_bin_files = config['docker_container'].getboolean('link_bin_files')
    else:
        link_bin_files = config['docker_container'].getboolean('link_bin_files')
    if not link_bin_files:
        return 0

    bin_dir = config['docker_container'].get('bin_dir', '')
    if not bin_dir:
        bin_dir = config['global'].get('bin_dir', '/usr/bin')

    link_deployed_bin_files(venv_directory, bin_dir)



if __name__ == '__main__':
    main()



#!/usr/bin/env python3
from __future__ import print_function
import logging
import os
import shutil
import sys

from configparser import ConfigParser, NoOptionError
from invirtualenv import __version__ as invirtualenv_version
from invirtualenv.deploy import build_deploy_virtualenv


def update_config(venv_dir, config_filename=None):
    if not config_filename:
        config_filename = 'deploy.conf'
    config = ConfigParser()
    config.read(config_filename)
    config.set('global', 'virtualenv_deploy_dir', venv_dir)
    with open(config_filename, 'w') as configfile:
        config.write(configfile)


def get_config_value(section, option, config_filename=None, default=''):
    if not config_filename:
        config_filename = 'deploy.conf'
    config = ConfigParser()
    config.read(config_filename)
    if default:
        return config[section].get(option, default)
    return config[section][option]


def get_config_flag(section, option, config_filename=None):
    if not config_filename:
        config_filename = 'deploy.conf'
    config = ConfigParser()
    config.read(config_filename)
    try:
        return config.getboolean(section, option)
    except NoOptionError:
        return


def main():
    log_level = logging.INFO
    if os.environ.get('RPM_SCRIPTLET_DEBUG', 'false').lower() in ['true', '1', 'on']:
        log_level = logging.DEBUG

    logging.basicConfig(level=log_level, format='%(asctime)s %(levelname)-5s [%(filename)s:%(lineno)d] %(message)s', datefmt='%Y%m%d:%H:%M:%S')
    logger = logging.getLogger(os.path.basename(sys.argv[0]))

    logger.debug('Starting rpm package post installprocessing using invirtualenv version %s' % invirtualenv_version)

    logger.debug('Running post install steps, from directory %r' % os.getcwd())
    upgrade = False
    scriptlet_argument = os.environ.get('RPM_ARG', '')
    if scriptlet_argument and scriptlet_argument == '2':
        upgrade = True
    logger.debug('Running post_install %s, upgrade=%r', scriptlet_argument, upgrade)

    if not os.path.exists('deploy.conf'):
        print("No 'deploy.conf' found.  Doing nothing", file=sys.stderr)
        sys.exit(1)

    logger.debug('Using the following deploy.conf: %s' % open('deploy.conf').read())
    venv_directory = None
    verbose = log_level == logging.DEBUG
    try:
        venv_directory = build_deploy_virtualenv(update_existing=True, verbose=verbose)
        update_config(venv_directory)
    except Exception:
        print('Unable to create the python virtualenv', file=sys.stderr)
        logger.exception('The virtualenv create failed, removing venv_directory')
        if venv_directory and os.path.exists(venv_directory):
            print('Removing the %r virtualenv directory' % venv_directory)
            shutil.rmtree(venv_directory)
        return 1

    print('Created virtualenv %r' % venv_directory)
    if get_config_flag('global', 'link_bin_files'):
        bin_dir = get_config_value('rpm_package', 'bin_dir', default='')
        if not bin_dir:
            bin_dir = get_config_value('global', 'bin_dir', default='/usr/bin')

        logger.debug('Linking files in the virtualenv bin directory to %r', os.path.dirname(sys.executable))
        try:
            from invirtualenv.deploy import link_deployed_bin_files
            link_deployed_bin_files(venv_directory, bin_dir)
            print('Linked bin files into the %s directory' % bin_dir)
        except ImportError:
            print('WARNING: The installed version of invirtualenv does not support linking bin files')

    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

import os
import json
from setuptools import setup


def scripts():
    """
    Get a list of the scripts in the scripts directory

    Returns
    -------
    list
        A list of strings containing the scripts in the scripts directory
    """
    script_list = []

    if os.path.isdir('scripts'):
        script_list += [
            os.path.join('scripts', f) for f in os.listdir('scripts')
        ]
    return script_list


if __name__ == '__main__':
    version = '1.0.0'
    setup(
        package_data= {
            'invirtualenv': [
                'package_metadata.json',
            ],
            'invirtualenv_plugins': [
                'docker_scripts/*'
            ]
        },
        scripts= scripts(),
    )



import logging
import os
import subprocess
from jinja2 import Template
import pkg_resources
import shutil

from invirtualenv.plugin_base import InvirtualenvPlugin
from invirtualenv.utility import csv_list, str_to_dict, find_executable


logger = logging.getLogger(__name__)


DOCKERFILE_TEMPLATE = """FROM {{docker_container['base_image']|default('ubuntu:17.10')}}

{% if docker_container['workdir_start'] %}WORKDIR {{docker_container['workdir_start']}}
{% endif -%}
{% if docker_container['add'] or docker_container['files'] or docker_container['copy'] %}# Files
{% endif -%}
{%- if docker_container['add'] %}{% for file_line in docker_container['add'] %}ADD {{file_line}}
{% endfor %}{% endif -%}
{%- if docker_container['files'] %}{% for file_line in docker_container['files'] %}COPY {{file_line}}
{% endfor %}{% endif -%}
{%- if docker_container['copy'] %}{% for file_line in docker_container['copy'] %}COPY {{file_line}}
{% endfor %}{% endif -%}
{%- if docker_container['add'] or docker_container['files'] or docker_container['copy'] %}
{% endif -%}
{% if docker_container['setenv'] or docker_container['env'] %}# Environment Settings
{% endif -%}{% if docker_container['setenv'] %}{% for setting, value in docker_container['setenv'].items() %}ENV {{setting}}="{{value}}"
{% endfor %}{% endif -%}
{%- if docker_container['env'] %}{% for setting, value in docker_container['env'].items() %}ENV {{setting}}="{{value}}"
{% endfor %}{% endif -%}
{%- if docker_container['env'] or docker_container['setenv'] %}
{% endif -%}
{% if docker_container['expose'] %}# Ports
{% for expose_port in docker_container['expose'] %}EXPOSE {{expose_port}}
{% endfor %}{% endif -%}
{%- if docker_container['expose'] %}
{% endif -%}
{% if docker_container['label'] %}# Labels
{% for label in docker_container['label'] %}LABEL {{label}}
{% endfor %}{% endif -%}
{%- if docker_container['label'] %}
{% endif -%}
{%- if docker_container['volume'] %}# Volumes
{% for volume in docker_container['volume'] %}VOLUME {{volume}}
{% endfor %}{% endif -%}
{%- if docker_container['volume'] %}
{% endif -%}
{%- if docker_container['run_before'] %}# Pre Invirtulenv Commands
{% for runline in docker_container['run_before'] %}RUN {{runline}}
{% endfor %}{% endif -%}
{%- if docker_container['run_before'] %}
{% endif -%}
# Set up invirtualenv in the container
ENV PATH="/var/lib/invirtualenv/installvenv/bin:${PATH}"
COPY docker_build.sh /tmp/docker_build.sh
COPY deploy.conf /var/lib/invirtualenv/deploy.conf
RUN chmod 755 /tmp/docker_build.sh && /tmp/docker_build.sh && rm /tmp/docker_build.sh

{% if docker_container['run_after'] %}# Post Invirtualenv Commands
{% for runline in docker_container['run_after'] %}RUN {{runline}}
{% endfor %}{% endif -%}
{%- if docker_container['run_after'] %}
{% endif -%}

{% if docker_container['entrypoint'] %}ENTRYPOINT {{docker_container['entrypoint']}}
{% endif -%}
{% if docker_container['cmd'] %}CMD {{docker_container['cmd']}}
{% endif -%}
{% if docker_container['healthcheck'] %}HEALTHCHECK {{docker_container['healthcheck']}}
{% endif -%}
{% if docker_container['stopsignal'] %}STOPSIGNAL {{docker_container['stopsignal']}}
{% endif -%}
{% if docker_container['user'] %}USER {{docker_container['user']}}
{% endif -%}
"""

DOCKER_CONFIG_DEFAULT = """[docker_container]
add=
base_image=ubuntu:17.10
cmd=
container_name=
copy:
entrypoint=
env:
expose:
deb_deps:
files:
healthcheck=
label:
rpm_deps:
run_before:
run_after:
setenv:
stopsignal=
user=
volume:
workdir_start=
"""


class InvirtualenvDocker(InvirtualenvPlugin):
    package_formats = ['docker']
    package_template = DOCKERFILE_TEMPLATE
    config_default = DOCKER_CONFIG_DEFAULT
    default_config_filename = 'Dockerfile.invirtualenv'
    config_types = {
        'docker_container': {
            'add': list,
            'base_image': str,
            'container_name': str,
            'cmd': str,
            'copy': list,
            'deb_deps': list,
            'entrypoint': str,
            'env': str_to_dict,
            'expose': list,
            'files': list,
            'healthcheck': str,
            'label': list,
            'rpm_deps': list,
            'run_before': list,
            'run_after': list,
            'setenv': str_to_dict,
            'stopsignal': str,
            'user': str,
            'volume': list,
            'workdir_start': str,
        }
    }

    def system_requirements_ok(self):
        if find_executable('docker'):
            return True
        logger.debug('The docker command is not present, disabling the docker plugin')
        return False

    def generate_wheel_archive(self, filename=None):
        # For docker containers we don't use wheels
        pass

    def write_command_scripts(self):
        shutil.copyfile(pkg_resources.resource_filename(__name__, 'docker_scripts/docker_build.sh'), 'docker_build.sh')

    def generate_wheel_packages(self, wheeldir):
        # For docker containers there is no need to generate and store wheels
        return {}

    def run_package_command(self, package_hashes, wheel_dir='wheels'):
        logger.debug('Config')
        logger.debug(self.config)
        if not self.config['docker_container'].get('container_name', None):
            self.config['docker_container']['container_name'] = 'invirtualenvapp/' + self.config['global']['name']
        self.config['docker_container']['files'].append('deploy.conf /var/lib/invirtualenv/deploy.conf')
        for package, hash in package_hashes.items():
            source_filename = os.path.join(wheel_dir, package)
            dest_filename = '/var/lib/invirtualenv/wheels/{name}/{package}'.format(name=self.config['global']['name'], package=package)
            self.config['docker_container']['files'].append(source_filename + ' ' + dest_filename)

        self.write_command_scripts()

        logger.debug('Dockerfile')
        logger.debug(self.render_template_with_config())
        with open('Dockerfile', 'w') as dockerfile_handle:
            dockerfile_handle.write(self.render_template_with_config())
        container_tag = '{name}:{version}'.format(name=self.config['docker_container']['container_name'], version=self.config['global']['version'])
        command = [find_executable('docker'), 'build', '-t', container_tag, '.']
        logger.debug('Running command %r', ' '.join(command))
        subprocess.check_call(command)
        logger.debug('Created container %r', container_tag)
        return container_tag



import os
import json
import logging
import shlex
import shutil
import subprocess

import distro
import pkgutil

from invirtualenv.plugin_base import InvirtualenvPlugin
from invirtualenv.utility import find_executable


logger = logging.getLogger(__name__)


SPEC_TEMPLATE = """Summary: {{global['description']|default('No summary available')}}
Name: {{global['name']}}
Version: {{global['version']|default('0.0.0')}}
Release: {{rpm_package['release']|default('1')}}
License: {{rpm_package['license']|default('Closed Source')}}
Group: {{rpm_package['group']|default('Development')}}
Packager: {{rpm_package['packager']|default('Verizon')}}
URL: {{global['url']|default('https://github.com/yahoo/invirtualenv')}}
AutoReqProv: no
{% if rpm_package['noarch'] %}BuildArch: noarch{% endif %}
{% if rpm_package['bootstrap_deps'] %}
# Install deps for {{ global['distro.name()'] }} {{ global['distro.major_version()'] }}.{{global['distro.minor_version()']}}
Requires(post): {% for package in rpm_package['bootstrap_deps'] %}{{package}}{{ ", " if not loop.last }}{% endfor %}
{% endif %}{% if rpm_package['deps'] %}
# RPM Package dependencies
Requires: {% for package in rpm_package['deps'] %}{{package}}{{ ", " if not loop.last }}{% endfor %}{% endif %}

%description
{{rpm_package['description']|default('No description')}}

%install
mkdir -p %{buildroot}/usr/share/%{name}_%{version}/
mkdir -p %{buildroot}/usr/share/%{name}_%{version}/package_scripts/
cp -r {{rpm_package['cwd']}}/wheels %{buildroot}/usr/share/%{name}_%{version}
cp {{rpm_package['cwd']}}/deploy.conf %{buildroot}/usr/share/%{name}_%{version}/deploy.conf
cp {{rpm_package['cwd']}}/post_install.py %{buildroot}/usr/share/%{name}_%{version}/package_scripts/post_install.py
cp {{rpm_package['cwd']}}/pre_uninstall.py %{buildroot}/usr/share/%{name}_%{version}/package_scripts/pre_uninstall.py
chmod 755 %{buildroot}/usr/share/%{name}_%{version}/package_scripts/post_install.py
chmod 755 %{buildroot}/usr/share/%{name}_%{version}/package_scripts/pre_uninstall.py
{% for source, dest in rpm_package['file_tuples'] %}mkdir -p $(dirname %{buildroot}{{dest[-1]}})
cp -a {{rpm_package['source_dir']}}/{{source}} %{buildroot}{{dest[-1]}}
{% endfor %}

%post
export RPM_ARG="$1"
export PATH=$PATH:/opt/python/bin:/usr/local/bin
export PIP_CMD="pip"

# Bootstrap a Python virtualenv with the invirtualenv utility deployed in it
echo "Attempting to build bootstrap venv using {{rpm_package['basepython']}}"
if [ ! -e "{{rpm_package['basepython']}}" ]; then
    echo "The python_interpreter was not found"
    exit 1
fi

{{rpm_package['basepython']}} -m venv "/usr/share/%{name}_%{version}/invirtualenv_deployer"
RC="$?"
if [ "$RC" != "0" ]; then
    virtualenv -p {{rpm_package['basepython']}} /usr/share/%{name}_%{version}/invirtualenv_deployer
fi

/usr/share/%{name}_%{version}/invirtualenv_deployer/bin/$PIP_CMD install --no-index --find-links=/usr/share/%{name}_%{version}/wheels invirtualenv configparser
RC="$?"
if [ "$RC" != "0" ]; then
    /usr/share/%{name}_%{version}/invirtualenv_deployer/bin/$PIP_CMD install --find-links=/usr/share/%{name}_%{version}/wheels invirtualenv configparser
fi

# Change into the directory containing this package's invirtualenv deployment configuration and run the invirtualenv_deployer
# to deploy the application in this rpm package.
cd /usr/share/%{name}_%{version}
/usr/share/%{name}_%{version}/invirtualenv_deployer/bin/python /usr/share/%{name}_%{version}/package_scripts/post_install.py

%preun
export RPM_ARG="$1"
/usr/share/%{name}_%{version}/invirtualenv_deployer/bin/python /usr/share/%{name}_%{version}/package_scripts/pre_uninstall.py

%postun
rm -rf /usr/share/%{name}_%{version}

%files
%defattr(0755, root, root, 0755)
/usr/share/%{name}_%{version}/*
{% for source, dest in rpm_package['file_tuples'] %}{% for elem in dest %}{{elem}} {% endfor %}
{% endfor %}
"""

RPM_CONFIG_DEFAULT = """[rpm_package]
bin_dir =
deps:
files:
"""

class InvirtualenvRPM(InvirtualenvPlugin):
    hash = 'sha256'
    package_formats = ['rpm']
    package_template = SPEC_TEMPLATE
    config_default = RPM_CONFIG_DEFAULT
    config_types = {
        'rpm_package': {
            'bin_dir': str,
            'deps': list,
            'files': list,
        }
    }
    default_config_filename = 'invirtualenv.spec'

    def add_plugin_configuration(self):
        # Make sure the configuration is sane
        # self.config['rpm_package']['deps'].append('invirtualenv')
        self.config['rpm_package']['cwd'] = os.getcwd()
        self.config['rpm_package']['source_dir'] = self.source_dir
        self.config['rpm_package']['noarch'] = self.noarch
        description = self.config['global'].get('description', '').strip()
        if not description:
            self.config['global']['description'] = 'No description available'

        basepython = self.config['rpm_package'].get('basepython', '').strip()
        gbasepython = self.config['global'].get('basepython', '').strip()
        if not basepython and gbasepython:
            self.config['rpm_package']['basepython'] = gbasepython
            self.config['global']['basepython'] = gbasepython

        # In some cases distro returns an empty string '' instead of 0, so we can't assume the value returned from
        # the calls to get that information is always an integer.
        try:
            major = int(distro.major_version())
        except ValueError:
            major = 0

        try:
            minor = int(distro.minor_version())
        except ValueError:
            with open('/etc/system-release') as fh:
                try:
                    minor = int(fh.read().strip().split()[3].split('.')[1])
                except (IndexError, ValueError):
                    minor = 0

        self.config['global']['distro.name()'] = distro.name()
        self.config['global']['distro.major_version()'] = major
        self.config['global']['distro.minor_version()'] = minor

        if major > 7 or (major == 7 and minor > 6):
            self.config['rpm_package']['bootstrap_deps'] = ['python3', 'python3-pip']  # RHEL 7.7 and newer
        else:
            self.config['rpm_package']['bootstrap_deps'] = ['python', 'python-pip', 'python-virtualenv']  # RHEL releases before 7.6


        # Parse the source and dest arguments in the files into a 2 item tuple
        self.config['rpm_package']['file_tuples'] = []
        for fileline in self.config['rpm_package']['files']:
            fileline = shlex.split(fileline)
            file_source_dest = fileline
            if len(fileline) == 1:
                file_source_dest = ('', fileline[0],)
            elif len(fileline) > 2:
                file_source_dest = (fileline[0], ' '.join(fileline[1:]), )
            if file_source_dest[1].startswith('%'):
                # The destination contains a directive, need to split it out
                temp = file_source_dest[1].split(')')
                directive = f'{temp[0]})'
                dest = ')'.join(temp[1:]).lstrip()
                file_source_dest[1] = [directive, dest]
            else:
                file_source_dest[1] = [file_source_dest[1]]

            self.config['rpm_package']['file_tuples'].append(file_source_dest)

    def copy_files_to_tempdir(self, tempdir):
        if 'file_tuples' not in self.config['rpm_package'].keys() or not self.config['rpm_package']['file_tuples']:
            return

        for source, dest in self.config['rpm_package']['file_tuples']:
            full_source = source
            full_dest = os.path.join(tempdir, dest[-1])
            if not source.startswith('/'):
                full_source = os.path.join(self.source_dir, source)
            if not os.path.exists(full_source):
                raise FileNotFoundError('[rpm_page] files entry %r not found' % full_source)
            logger.debug('copying', full_source, full_dest)
            os.makedirs(os.path.dirname(full_dest), exist_ok=True)
            shutil.copyfile(full_source, full_dest)

    def system_requirements_ok(self):
        if find_executable('rpmbuild'):
            return True
        logger.debug('The rpmbuild command is not present, disabling the rpm plugin')
        return False

    def run_package_command(self, package_hashes, wheel_dir='wheels'):
        self.config['rpm_package']['cwd'] = os.getcwd()

        # Get the packaging script
        for script in ['rpm_scripts/post_install.py', 'rpm_scripts/pre_uninstall.py']:
            with open(os.path.basename(script), 'wb') as script_handle:
                script_handle.write(pkgutil.get_data('invirtualenv_plugins', script))

        logger.debug('Config')
        logger.debug(json.dumps(self.config, indent=4, sort_keys=True))
        logger.debug('Spec')
        logger.debug(self.render_template_with_config())
        logger.debug('Deploy.conf')
        logger.debug(open('deploy.conf').read())
        logger.debug('CWD: %s', os.getcwd())
        logger.debug('source_dir: %s', self.source_dir)
        with open('package.spec', 'w') as spec_handle:
            spec_handle.write(self.render_template_with_config())
        command = [find_executable('rpmbuild'), '-ba', 'package.spec']
        logger.debug('Running command %r', ' '.join(command))
        output = subprocess.check_output(command, env={'LANG': 'C'})
        output = output.decode(errors='ignore')
        packages = []
        for line in output.split('\n'):
            line = line.strip()
            if line.startswith('Wrote: '):
                packages.append(line.split()[-1])
        logger.debug('found packages %r', packages)
        if packages:
            return packages[-1]
